
from ctypes import *
from ctypes.util import find_library
import cv2

from blc_array import BlcArray
path=find_library("blc")
lib = CDLL(name=path)

class BlcArrayTCP4Server(BlcArray):
    lib.blc_array_get_type.argtypes = [ c_void_p ]
    lib.blc_array_get_type.restype = c_uint32
    lib.blc_array_tcp4_server_new_void.restype = c_void_p
    lib.blc_array_tcp4_server_new_void.argtypes = [ c_char_p ]
    lib.blc_array_tcp4_server_new.restype = c_void_p

    lib.blc_array_tcp4_server_recv_data.argtypes = [ c_void_p ]
    lib.blc_array_tcp4_server_send_data.argtypes = [ c_void_p ]
    lib.blc_array_tcp4_server_wait_connection.argtypes = [ c_void_p ]
    lib.blc_array_tcp4_server_wait_connection.restype = c_int 
    lib.blc_array_tcp4_server_delete.argtypes = [ c_void_p ]
        
    def __init__(self, port, data_type=None, format_type=None, dims=None):
        self.port = port
        if (dims):
            dim_types = [c_size_t] * len(dims)
            lib.blc_array_tcp4_server_new.argtypes = [ c_char_p, c_uint32, c_uint32, c_int, *dim_types ]
            type_int =   int.from_bytes(data_type.encode(), "big")
            format_int = int.from_bytes(format_type.encode(), "big")
            self.pointer = lib.blc_array_tcp4_server_new(port.encode(), type_int, format_int, len(dims), *dims )
        else:
            self.pointer = lib.blc_array_tcp4_server_new_void(port.encode())        

        self.get_def()

    def send_data(self):
        lib.blc_array_tcp4_server_send_data(self.pointer)
    
    def recv_data(self):
        lib.blc_array_tcp4_server_recv_data(self.pointer)
        
    def wait_connection(self):
        error = lib.blc_array_tcp4_server_wait_connection(self.pointer)
        if (error != 0):
            raise Exception('Error in accept:'+ error)
        self.get_def() # update the array definition with the c data
    
    def __del__(self):
        lib.blc_array_tcp4_server_delete(self.pointer)

class BlcArrayTCP4Client(BlcArray):
    lib.blc_array_tcp4_client_new_void.restype = c_void_p
    lib.blc_array_tcp4_client_new_void.argtypes = [ c_char_p, c_char_p ]
    
    lib.blc_array_tcp4_client_new.restype = c_void_p

    lib.blc_array_tcp4_client_recv_data.argtypes = [ c_void_p ]
    lib.blc_array_tcp4_client_recv_data.restype = c_int
    
    lib.blc_array_tcp4_client_bytes_to_read.argtypes = [ c_void_p ]
    lib.blc_array_tcp4_client_bytes_to_read.restype = c_int

    lib.blc_array_tcp4_client_send_data.argtypes = [ c_void_p ]
    lib.blc_array_tcp4_client_delete.argtypes = [ c_void_p ]

    def __init__(self, address, port, data_type=None, format_type=None, dims=None):
        self.address=address
        self.port = port
        
        if dims:
            dim_types = [c_size_t] * len(dims)
            lib.blc_array_tcp4_client_new.argtypes = [ c_char_p, c_char_p, c_uint32, c_uint32, c_int, *dim_types ]
            self.pointer = lib.blc_array_tcp4_client_new( address.encode(), port.encode(),  int.from_bytes(data_type.encode(), "big"), int.from_bytes(format_type.encode(), "big"), len(dims), *dims )
        else:
            self.pointer = lib.blc_array_tcp4_client_new_void(address.encode(), port.encode())
        
        self.get_def() #get def of the array from the c data
        

    
    def send_data(self):
        lib.blc_array_tcp4_client_send_data(self.pointer)
    
    def recv_data(self):
        return lib.blc_array_tcp4_client_recv_data(self.pointer)
    
    def bytes_to_recv(self):
        return lib.blc_array_tcp4_client_bytes_to_read(self.pointer)
    
    def __del__(self):
        lib.blc_array_tcp4_client_delete(self.pointer)
   
                
if __name__ == "__main__":
    import threading
    
    '''
    This thread is only for example, it is supposed to be executed in another process ( the server )
    '''
    '''Client part'''
    class ServerThread (threading.Thread):
        def run(self):
            server = BlcArrayTCP4Server( "31440", 'UIN8', 'TEXT', [5])
            print("server data before change:", server.data[:])
            server.data[3]=7 #Change values to check if the data will be received
            print("server data after change:", server.data[:])
            print("Server has started, waiting for connection\n")
            server.wait_connection()
            print("Client connected")
            server.send_data()
    
    print("Initialize server thread")
    thread=ServerThread()
    thread.start()
    

    '''Client part'''
    client = BlcArrayTCP4Client("localhost", "31440")
    print("Values before receiving the data")
    print(client.data[:])
    client.recv_data()
    print("Values after receiving the data")
    print(client.data[:])
    thread.join()
    #On some IDEs the __del__ functions are not called which cause the socket not being stopped before one minute    
    
    
